#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import json
import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.varios import *
from ikabot.helpers.process import forkear
from ikabot.helpers.varios import addPuntos
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.recursos import getRecursosDisponibles

t = gettext.translation('entrenarTropas',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def getCuartelInfo(s, ciudad):
	params = {'view': 'barracks', 'cityId': ciudad['id'], 'position': ciudad['pos'], 'backgroundView': 'city', 'currentCityId': ciudad['id'], 'actionRequest': s.token(), 'ajax': '1'}
	data = s.post(params=params)
	return json.loads(data, strict=False)

def entrenar(s, ciudad, entrenamiento):
	payload = {'action': 'CityScreen', 'function': 'buildUnits', 'actionRequest': s.token(), 'cityId': ciudad['id'], 'position': ciudad['pos'], 'backgroundView': 'city', 'currentCityId': ciudad['id'], 'templateView': 'barracks', 'ajax': '1'}
	for tropa in entrenamiento:
		payload[ tropa['unit_type_id'] ] = tropa['entrenar']
	s.post(payloadPost=payload)

def esperarEntrenamiento(s, ciudad):
	data = getCuartelInfo(s, ciudad)
	html = data[1][1][1]
	segundos = re.search(r'\'buildProgress\', (\d+),', html)
	if segundos:
		segundos = segundos.group(1)
		segundos = int(segundos) - data[0][1]['time']
		esperar(segundos + 5)

def getCiudadanosDisponibles(html):
	ciudadanosDisp = re.search(r'js_GlobalMenu_citizens">(.*?)</span>', html).group(1)
	return int(ciudadanosDisp.replace(',', ''))

def planearEntrenamientos(s, ciudad, entrenamientos):
	while True:
		total = 0
		for entrenamiento in entrenamientos:
			for tropa in entrenamiento:
				total += tropa['cantidad']
		if total == 0:
			return

		for entrenamiento in entrenamientos:
			esperarEntrenamiento(s, ciudad)
			html = s.get(urlCiudad + ciudad['id'])
			ciudadanosDisp = getCiudadanosDisponibles(html)
			recursos = getRecursosDisponibles(html, num=True)
			maderaDisp  = recursos[0]
			vinoDisp    = recursos[1]
			marmolDisp  = recursos[2]
			cristalDisp = recursos[3]
			azufreDisp  = recursos[4]

			for tropa in entrenamiento:

				tropa['entrenar'] = tropa['cantidad']

				if 'wood' in tropa['costs']:
					limitante = maderaDisp // tropa['costs']['wood']
					if limitante < tropa['entrenar']:
						tropa['entrenar'] = limitante
				if 'wine' in tropa['costs']:
					limitante = vinoDisp // tropa['costs']['wine']
					if limitante < tropa['entrenar']:
						tropa['entrenar'] = limitante
				if 'marble' in ['costs']:
					limitante = marmolDisp // tropa['costs']['marble']
					if limitante < tropa['entrenar']:
						tropa['entrenar'] = limitante
				if 'cristal' in ['costs']:
					limitante = cristalDisp // tropa['costs']['cristal']
					if limitante < tropa['entrenar']:
						tropa['entrenar'] = limitante
				if 'sulfur' in tropa['costs']:
					limitante = azufreDisp // tropa['costs']['sulfur']
					if limitante < tropa['entrenar']:
						tropa['entrenar'] = limitante
				if 'citizens' in tropa['costs']:
					limitante = ciudadanosDisp // tropa['costs']['citizens']
					if limitante < tropa['entrenar']:
						tropa['entrenar'] = limitante

				if 'wood' in tropa['costs']:
					maderaDisp -= tropa['costs']['wood'] * tropa['entrenar']
				if 'wine' in tropa['costs']:
					vinoDisp -= tropa['costs']['wine'] * tropa['entrenar']
				if 'marble' in tropa['costs']:
					marmolDisp -= tropa['costs']['marble'] * tropa['entrenar']
				if 'cristal' in ['costs']:
					cristalDisp -= tropa['costs']['cristal'] * tropa['entrenar']
				if 'sulfur' in tropa['costs']:
					azufreDisp -= tropa['costs']['sulfur'] * tropa['entrenar']
				if 'citizens' in tropa['costs']:
					ciudadanosDisp -= tropa['costs']['citizens'] * tropa['entrenar']

				tropa['cantidad'] -= tropa['entrenar']

			total = 0
			for tropa in entrenamiento:
				total += tropa['entrenar']
			if total == 0:
				msg = _('No se pudo terminar de entrenar tropas por falta de recursos.')
				sendToBot(msg)
				return
			entrenar(s, ciudad, entrenamiento)

def generateTroops(unidades_info):
	i = 1
	unidades = []
	while 'js_barracksSlider{:d}'.format(i) in unidades_info:
		# {"identifier":"phalanx","unit_type_id":303,"costs":{"citizens":1,"wood":27,"sulfur":30,"upkeep":3,"completiontime":71.169695412658},"local_name":"Hoplita"}
		info = unidades_info['js_barracksSlider{:d}'.format(i)]['slider']['control_data']
		info = json.loads(info, strict=False)
		unidades.append(info)
		i += 1
	return unidades

def entrenarTropas(s):
	banner()
	print(_('¿En qué ciudad quiere entrenar las tropas?'))
	ciudad = elegirCiudad(s)
	banner()

	for i in range(len(ciudad['position'])):
		if ciudad['position'][i]['building'] == 'barracks':
			ciudad['pos'] = str(i)
			break

	data = getCuartelInfo(s, ciudad)
	unidades_info = data[2][1]
	unidades = generateTroops(unidades_info)

	maxSize = 0
	for unidad in unidades:
		if maxSize < len(unidad['local_name']):
			maxSize = len(unidad['local_name'])

	entrenamientos = []
	while True:
		unidades = generateTroops(unidades_info)
		print(_('Entrenar:'))
		for unidad in unidades:
			cantidad = read(msg='{}{}:'.format(' '*(maxSize-len(unidad['local_name'])), unidad['local_name']), min=0, empty=True)
			if cantidad == '':
				cantidad = 0
			unidad['cantidad'] = cantidad

		print(_('\nCosto total:'))
		costo = {'madera': 0, 'vino': 0, 'marmol': 0, 'cristal': 0, 'azufre': 0, 'ciudadanos': 0, 'manuntencion': 0, 'tiempo': 0}
		for unidad in unidades:

			if 'wood' in unidad['costs']:
				costo['madera'] += unidad['costs']['wood'] * unidad['cantidad']
			if 'wine' in unidad['costs']:
				costo['vino'] += unidad['costs']['wine'] * unidad['cantidad']
			if 'marble' in unidad['costs']:
				costo['marmol'] += unidad['costs']['marble'] * unidad['cantidad']
			if 'cristal' in unidad['costs']:
				costo['cristal'] += unidad['costs']['cristal'] * unidad['cantidad']
			if 'sulfur' in unidad['costs']:
				costo['azufre'] += unidad['costs']['sulfur'] * unidad['cantidad']
			if 'citizens' in unidad['costs']:
				costo['ciudadanos'] += unidad['costs']['citizens'] * unidad['cantidad']
			if 'upkeep' in unidad['costs']:
				costo['manuntencion'] += unidad['costs']['upkeep'] * unidad['cantidad']
			if 'completiontime' in unidad['costs']:
				costo['tiempo'] += unidad['costs']['completiontime'] * unidad['cantidad']

		if costo['madera']:
			print(_('     Madera: {}').format(addPuntos(costo['madera'])))
		if costo['vino']:
			print(_('       Vino: {}').format(addPuntos(costo['vino'])))
		if costo['marmol']:
			print(_('     Marmol: {}').format(addPuntos(costo['marmol'])))
		if costo['cristal']:
			print(_('    Cristal: {}').format(addPuntos(costo['cristal'])))
		if costo['azufre']:
			print(_('     Azufre: {}').format(addPuntos(costo['azufre'])))
		if costo['ciudadanos']:
			print(_(' Ciudadanos: {}').format(addPuntos(costo['ciudadanos'])))
		if costo['manuntencion']:
			print(_('Manutención: {}').format(addPuntos(costo['manuntencion'])))
		if costo['tiempo']:
			print(_('   Duración: {}').format(diasHorasMinutos(int(costo['tiempo']))))

		print(_('\nProceder? [Y/n]'))
		rta = read(values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'n':
			return

		entrenamientos.append(unidades)

		print(_('\n¿Quiere entrenar más tropas al terminar? [y/N]'))
		rta = read(values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'y':
			banner()
			continue
		else:
			break

	recursos   = getRecursosDisponibles( ciudad['html'], num=True )
	ciudadanos = getCiudadanosDisponibles( ciudad['html'] )
	sobrante               = {}
	sobrante['madera']     = recursos[0]
	sobrante['vino']       = recursos[1]
	sobrante['marmol']     = recursos[2]
	sobrante['cristal']    = recursos[3]
	sobrante['azufre']     = recursos[4]
	sobrante['ciudadanos'] = ciudadanos

	for entrenamiento in entrenamientos:
		for unidad in entrenamiento:

			if 'wood' in unidad['costs']:
				sobrante['madera'] -= unidad['costs']['wood'] * unidad['cantidad']
			if 'wine' in unidad['costs']:
				sobrante['vino'] -= unidad['costs']['wine'] * unidad['cantidad']
			if 'marble' in unidad['costs']:
				sobrante['marmol'] -= unidad['costs']['marble'] * unidad['cantidad']
			if 'cristal' in unidad['costs']:
				sobrante['cristal'] -= unidad['costs']['cristal'] * unidad['cantidad']
			if 'sulfur' in unidad['costs']:
				sobrante['azufre'] -= unidad['costs']['sulfur'] * unidad['cantidad']
			if 'citizens' in unidad['costs']:
				sobrante['ciudadanos'] -= unidad['costs']['citizens'] * unidad['cantidad']

	falta = [ elem for elem in sobrante if sobrante[elem] < 0 ] != []

	if falta:
		print(_('\nNo hay suficientes recursos:'))
		if sobrante['madera'] < 0:
			print(_('    Madera:{}').format(addPuntos(sobrante['madera']*-1)))
		if sobrante['vino'] < 0:
			print(_('      Vino:{}').format(addPuntos(sobrante['vino']*-1)))
		if sobrante['marmol'] < 0:
			print(_('    Marmol:{}').format(addPuntos(sobrante['marmol']*-1)))
		if sobrante['cristal'] < 0:
			print(_('   Cristal:{}').format(addPuntos(sobrante['cristal']*-1)))
		if sobrante['azufre'] < 0:
			print(_('    Azufre:{}').format(addPuntos(sobrante['azufre']*-1)))
		if sobrante['ciudadanos'] < 0:
			print(_('Ciudadanos:{}').format(addPuntos(sobrante['ciudadanos']*-1)))

		print(_('\n¿Proceder de todos modos? [Y/n]'))
		rta = read(values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'n':
			return

	print(_('\nSe entrenarán las tropas seleccionadas.'))
	enter()

	forkear(s)
	if s.padre is True:
		return

	info = _('\nEntreno tropas en {}\n').format(ciudad['cityName'])
	setInfoSignal(s, info)
	try:
		planearEntrenamientos(s, ciudad, entrenamientos)
	except:
		msg = _('Error en:\n{}\nCausa:\n{}').format(info, traceback.format_exc())
		sendToBot(msg)
	finally:
		s.logout()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.getJson import getCiudad

t = gettext.translation('modoVacaciones',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def activarModoVacaciones(s):
	html = s.get()
	ciudad = getCiudad(html)

	data = {'action': 'Options', 'function': 'activateVacationMode', 'actionRequest': s.token(), 'backgroundView': 'city', 'currentCityId': ciudad['id'], 'templateView': 'options_umod_confirm'}
	s.post(params=data, ignoreExpire=True)

def modoVacaciones(s):
	banner()
	print(_('¿Activar modo vacaciones? [Y/n]'))
	rta = read(values=['y', 'Y', 'n', 'N', ''])
	if rta.lower() == 'n':
		return

	activarModoVacaciones(s)

	print(_('Se activo el modo vacaciones.'))
	enter()
	clear()
	exit()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.planearViajes import planearViajes
from ikabot.helpers.recursos import getRecursosDisponibles
from ikabot.helpers.varios import addPuntos
from ikabot.helpers.process import forkear
from ikabot.helpers.gui import banner


def enviarVino(s):
	t = gettext.translation('enviarVino', 
	                        localedir, 
	                        languages=idiomas,
	                        fallback=True)
	_ = t.gettext
	banner()
	vinoTotal = 0
	dict_idVino_diponible = {}
	(idsCiudades, ciudades) = getIdsDeCiudades(s)
	ciudadesVino = {}
	for idCiudad in idsCiudades:
		esVino =  ciudades[idCiudad]['tradegood'] == '1'
		if esVino:
			html = s.get(urlCiudad + idCiudad)
			ciudad = getCiudad(html)
			recursos = getRecursosDisponibles(html)
			disponible = int(recursos[1]) - 1000 # dejo 1000 por las dudas
			ciudad['disponible'] = disponible if disponible > 0 else 0
			vinoTotal += ciudad['disponible']
			ciudadesVino[idCiudad] = ciudad
	aEnviar = len(ciudades) - len(ciudadesVino)
	vinoXciudad = int(vinoTotal / aEnviar)
	maximo = addPuntos(vinoXciudad)

	if vinoXciudad > 1000:
		maximo = maximo[:-3] + '000'

	print(_('Se puede enviar como máximo {} a cada ciudad').format(maximo))
	cantidad = read(msg=_('¿Cuanto vino enviar a cada ciudad?:'), min=0, max=vinoXciudad)

	if cantidad == 0:
		return

	print(_('\nPor enviar {} de vino a cada ciudad').format(addPuntos(cantidad)))
	print(_('¿Proceder? [Y/n]'))
	rta = read(values=['y', 'Y', 'n', 'N', ''])
	if rta.lower() == 'n':
		return

	forkear(s)
	if s.padre is True:
		return

	rutas = []
	for idCiudadDestino in [ idCity for idCity in idsCiudades if idCity not in ciudadesVino ]:
		htmlD = s.get(urlCiudad + idCiudadDestino)
		ciudadD = getCiudad(htmlD)
		idIsla = ciudadD['islandId']
		faltante = cantidad
		for idCiudadOrigen in ciudadesVino:
			if faltante == 0:
				break
			ciudadO = ciudadesVino[idCiudadOrigen]
			vinoDisponible = ciudadO['disponible']
			for ruta in rutas:
				(origen, __, __, __, vn, __, __, __) = ruta
				if origen['id'] == idCiudadOrigen:
					vinoDisponible -= vn
			enviar = faltante if vinoDisponible > faltante else vinoDisponible
			faltante -= enviar
			ruta = (ciudadO, ciudadD, idIsla, 0, enviar, 0, 0, 0)
			rutas.append(ruta)

	info = _('\nEnviar vino\n')
	for ruta in rutas:
		(ciudadO, ciudadD, idIsla, md, vn, mr, cr, az) = ruta
		info = info + _('{} -> {}\nVino: {}\n').format(ciudadO['cityName'], ciudadD['cityName'], addPuntos(vn))
	setInfoSignal(s, info)
	try:
		planearViajes(s, rutas)
	except:
		msg = _('Error en:\n{}\nCausa:\n{}').format(info, traceback.format_exc())
		sendToBot(msg)
	finally:
		s.logout()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re

def getRecursosDisponibles(html, num=False):
	recursos = re.search(r'\\"resource\\":(\d+),\\"2\\":(\d+),\\"1\\":(\d+),\\"4\\":(\d+),\\"3\\":(\d+)}', html)
	if num:
		return [int(recursos.group(1)), int(recursos.group(3)), int(recursos.group(2)), int(recursos.group(5)), int(recursos.group(4))]
	else:
		return [recursos.group(1), recursos.group(3), recursos.group(2), recursos.group(5), recursos.group(4)]

def getCapacidadDeAlmacenamiento(html, num=False):
	almacenamiento = re.search(r'maxResources:\s*JSON\.parse\(\'{\\"resource\\":(\d+),', html).group(1)
	if num:
		almacenamiento = int(almacenamiento)
	return almacenamiento 

def getConsumoDeVino(html):
	return int(re.search(r'GlobalMenu_WineConsumption"\s*class="rightText">\s*(\d*)\s', html).group(1))



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import re

def borrar(texto, ocurrencias):
	for ocurrencia in ocurrencias:
		texto = texto.replace(ocurrencia, '')
	return texto

def getIsla(html):
	isla = re.search(r'\[\["updateBackgroundData",([\s\S]*?),"specialServerBadges', html).group(1) + '}'

	isla = isla.replace('buildplace', 'empty')
	isla = isla.replace('xCoord', 'x')
	isla = isla.replace('yCoord', 'y')
	isla = isla.replace(',"owner', ',"')
	isla = isla.replace(',"tradegoodLevel',',"goodLv')
	isla = isla.replace(',"tradegood', ',"good')
	isla = isla.replace(',"resourceLevel', ',"woodLv')
	isla = isla.replace(',"wonderLevel', ',"wonderLv')
	isla = isla.replace('avatarScores', 'scores')

	remove = []

	sub = re.search(r',"type":\d', isla).group()
	remove.append(sub)

	quitar = re.search(r'(,"barbarians[\s\S]*?),"scores"', isla).group(1)

	remove.append(quitar)
	remove.append(',"goodTarget":"tradegood"')
	remove.append(',"name":"Building ground"')
	remove.append(',"name":"Terreno"')
	remove.append(',"actions":[]')
	remove.append('"id":-1,')
	remove.append(',"level":0,"viewAble":1')
	remove.append(',"empty_type":"normal"')
	remove.append(',"empty_type":"premium"')
	remove.append(',"hasTreaties":0')
	remove.append(',"hasTreaties":1')
	remove.append(',"infestedByPlague":false')
	remove.append(',"infestedByPlague":true')
	remove.append(',"viewAble":0')
	remove.append(',"viewAble":1')
	remove.append(',"viewAble":2')
	isla = borrar(isla, remove)
	# {"id":idIsla,"name":nombreIsla,"x":,"y":,"good":numeroBien,"woodLv":,"goodLv":,"wonder":numeroWonder, "wonderName": "nombreDelMilagro","wonderLv":"5","cities":[{"type":"city","name":cityName,"id":cityId,"level":lvIntendencia,"Id":playerId,"Name":playerName,"AllyId":,"AllyTag":,"state":"vacation"},...}}
	return json.loads(isla, strict=False)

def getCiudad(html):
	ciudad = re.search(r'"updateBackgroundData",([\s\S]*?),"(?:beachboys|spiesInside)', html).group(1) + '}'

	ciudad = ciudad.replace(',"owner', ',"')
	ciudad = ciudad.replace('islandXCoord','x')
	ciudad = ciudad.replace('islandYCoord','y')
	ciudad = '{"cityName"' + ciudad[len('{"name"'):]

	remove = []

	sub = re.search(r',"buildingSpeedupActive":\d', ciudad)
	remove.append(sub.group())

	sub = re.search(r',"showPirateFortressBackground":\d', ciudad)
	remove.append(sub.group())

	sub = re.search(r',"showPirateFortressShip":\d', ciudad)
	remove.append(sub.group())

	ciudad = borrar(ciudad, remove)

	for elem in ['sea', 'land', 'shore', 'wall']:
		ciudad = ciudad.replace('"building":"buildingGround {}"'.format(elem),'"name":"empty","building":"empty"')
	ciudad = ciudad.replace('"isBusy":true,','"isBusy":false,')

	ampliando = re.findall(r'(("name":"[\w\s\\]*","level":"\d*","isBusy":false,"canUpgrade":\w*,"isMaxLevel":\w*,"building":"\w*?)\sconstructionSite","(?:completed|countdownText|buildingimg).*?)}',ciudad)
	for edificio in ampliando:
		viejo = edificio[1]+'"'
		nuevo = viejo.replace('"isBusy":false,', '"isBusy":true,')
		ciudad = ciudad.replace(edificio[0], nuevo)

	# {'cityName': '', 'id': 'idCiudad', 'phase': 5, 'isCapital': True|False, 'Id': 'idJugador', 'Name': 'nombreJugador', 'islandId': 'idIsla', 'islandName': 'NombreIsla', 'x': 'Coordx', 'y': 'Coordy', 'underConstruction': -1, 'endUpgradeTime': -1, 'startUpgradeTime': -1, 'position': [{'name': 'nombreEdificio', 'level': 'nivel', 'isBusy': True|False, 'canUpgrade': True|False, 'isMaxLevel': True|False, 'building': 'nombreEnIngles'}, ...]}
	ciudad = json.loads(ciudad, strict=False)
	ciudad['html'] = html
	ciudad['propia'] = True
	return ciudad



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import math
import json
import gettext
import traceback
from decimal import *
from ikabot.helpers.process import forkear
from ikabot.helpers.varios import addPuntos
from ikabot.helpers.gui import enter, banner
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.planearViajes import esperarLlegada
from ikabot.helpers.pedirInfo import getIdsDeCiudades, read
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.recursos import *

t = gettext.translation('comprarRecursos', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def asignarRecursoBuscado(s, ciudad):
	print(_('¿Qué recurso quiere comprar?'))
	for indice, bien in enumerate(tipoDeBien):
		print('({:d}) {}'.format(indice+1, bien))
	eleccion = read(min=1, max=5)
	recurso = eleccion - 1
	if recurso == 0:
		recurso = 'resource'
	data = {
	'cityId': ciudad['id'],
	'position': ciudad['pos'],
	'view': 'branchOffice',
	'activeTab': 'bargain',
	'type': 444,
	'searchResource': recurso,
	'range': ciudad['rango'],
	'backgroundView' : 'city',
	'currentCityId': ciudad['id'],
	'templateView': 'branchOffice',
	'currentTab': 'bargain',
	'actionRequest': s.token(),
	'ajax': 1
	}
	rta = s.post(payloadPost=data)
	return eleccion, recurso

def getStoreHtml(s, ciudad):
	url = 'view=branchOffice&cityId={}&position={:d}&currentCityId={}&backgroundView=city&actionRequest={}&ajax=1'.format(ciudad['id'], ciudad['pos'], ciudad['id'], s.token())
	data = s.post(url)
	json_data = json.loads(data, strict=False)
	return json_data[1][1][1]

def obtenerOfertas(s, ciudad):
	html = getStoreHtml(s, ciudad)
	hits = re.findall(r'short_text80">(.*?) *<br/>\((.*?)\)\s *</td>\s *<td>(\d+)</td>\s *<td>(.*?)/td>\s *<td><img src="skin/resources/icon_(\w+)\.png[\s\S]*?white-space:nowrap;">(\d+)\s[\s\S]*?href="\?view=takeOffer&destinationCityId=(\d+)&oldView=branchOffice&activeTab=bargain&cityId=(\d+)&position=(\d+)&type=(\d+)&resource=(\w+)"', html)
	ofertas = []
	for hit in hits:
		oferta = {
		'ciudadDestino': hit[0],
		'jugadorAComprar' : hit[1],
		'bienesXminuto': int(hit[2]),
		'cantidadDisponible': int(hit[3].replace(',', '').replace('<', '')),
		'tipo': hit[4],
		'precio': int(hit[5]),
		'destinationCityId': hit[6],
		'cityId': hit[7],
		'position': hit[8],
		'type': hit[9],
		'resource': hit[10]
		}
		ofertas.append(oferta)
	return ofertas

def calcularCosto(ofertas, cantidadAComprar):
	costoTotal = 0
	for oferta in ofertas:
		if cantidadAComprar == 0:
			break
		comprar = oferta['cantidadDisponible'] if oferta['cantidadDisponible'] < cantidadAComprar else cantidadAComprar
		cantidadAComprar -= comprar
		costoTotal += comprar * oferta['precio']
	return costoTotal

def getOro(s, ciudad):
	url = 'view=finances&backgroundView=city&currentCityId={}&templateView=finances&actionRequest={}&ajax=1'.format(ciudad['id'], s.token())
	data = s.post(url)
	json_data = json.loads(data, strict=False)
	oro = json_data[0][1]['headerData']['gold']
	return int(oro.split('.')[0])

def getCiudadesComerciales(s):
	ids = getIdsDeCiudades(s)[0]
	ciudades_comerciales = []
	for idCiudad in ids:
		html = s.get(urlCiudad + idCiudad)
		ciudad = getCiudad(html)
		for pos, edificio in enumerate(ciudad['position']):
			if edificio['building'] == 'branchOffice':
				ciudad['pos'] = pos
				html = getStoreHtml(s, ciudad)
				rangos = re.findall(r'<option.*?>(\d+)</option>', html)
				ciudad['rango'] = int(rangos[-1])
				ciudades_comerciales.append(ciudad)
				break
	return ciudades_comerciales

def comprarRecursos(s):
	banner()

	ciudades_comerciales = getCiudadesComerciales(s)
	if len(ciudades_comerciales) == 0:
		print(_('No hay una Tienda contruida'))
		enter()
		return

	ciudad = ciudades_comerciales[0] # por ahora solo uso la primera ciudad

	numRecurso, recurso = asignarRecursoBuscado(s, ciudad)
	banner()

	ofertas = obtenerOfertas(s, ciudad)
	if len(ofertas) == 0:
		print(_('No se encontraron ofertas.'))
		return

	precio_total   = 0
	cantidad_total = 0
	for oferta in ofertas:
		cantidad = oferta['cantidadDisponible']
		unidad   = oferta['precio']
		costo    = cantidad * unidad
		print(_('cantidad :{}').format(addPuntos(cantidad)))
		print(_('precio   :{:d}').format(unidad))
		print(_('costo    :{}').format(addPuntos(costo)))
		print('')
		precio_total += costo
		cantidad_total += cantidad

	ocupado = getRecursosDisponibles(ciudad['html'], num=True)[numRecurso - 1]
	capacidad = getCapacidadDeAlmacenamiento(ciudad['html'], num=True)
	disponible = capacidad - ocupado

	print(_('Total disponible para comprar: {}, por {}').format(addPuntos(cantidad_total), addPuntos(precio_total)))
	if disponible < cantidad_total:
		print(_('Solo se puede comprar {} por falta de almacenamiento.').format(addPuntos(disponible)))
		cantidad_total = disponible
	print('')
	cantidadAComprar = read(msg=_('¿Cuánta cantidad comprar? '), min=0, max=cantidad_total)
	if cantidadAComprar == 0:
		return

	oro = getOro(s, ciudad)
	costoTotal = calcularCosto(ofertas, cantidadAComprar)

	print(_('\nOro actual : {}.\nCosto total: {}.\nOro final  : {}.'). format(addPuntos(oro), addPuntos(costoTotal), addPuntos(oro - costoTotal)))
	print(_('¿Proceder? [Y/n]'))
	rta = read(values=['y', 'Y', 'n', 'N', ''])
	if rta.lower() == 'n':
		return

	print(_('Se comprará {}').format(addPuntos(cantidadAComprar)))
	enter()

	forkear(s)
	if s.padre is True:
		return

	info = _('\nCompro {} de {} para {}\n').format(addPuntos(cantidadAComprar), tipoDeBien[numRecurso - 1], ciudad['cityName'])
	setInfoSignal(s, info)
	try:
		do_it(s, ciudad, ofertas, cantidadAComprar)
	except:
		msg = _('Error en:\n{}\nCausa:\n{}').format(info, traceback.format_exc())
		sendToBot(msg)
	finally:
		s.logout()

def buy(s, ciudad, oferta, cantidad):
	barcos = int(math.ceil((Decimal(cantidad) / Decimal(500))))
	data_dict = {
	'action': 'transportOperations',
	'function': 'buyGoodsAtAnotherBranchOffice',
	'cityId': oferta['cityId'],
	'destinationCityId': oferta['destinationCityId'],
	'oldView': 'branchOffice',
	'position': ciudad['pos'],
	'avatar2Name': oferta['jugadorAComprar'],
	'city2Name': oferta['ciudadDestino'],
	'type': int(oferta['type']),
	'activeTab': 'bargain',
	'transportDisplayPrice': 0,
	'premiumTransporter': 0,
	'capacity': 5,
	'max_capacity': 5,
	'jetPropulsion': 0,
	'transporters': barcos,
	'backgroundView': 'city',
	'currentCityId': oferta['cityId'],
	'templateView': 'takeOffer',
	'currentTab': 'bargain',
	'actionRequest': s.token(),
	'ajax': 1
	}
	url = 'view=takeOffer&destinationCityId={}&oldView=branchOffice&activeTab=bargain&cityId={}&position={}&type={}&resource={}&backgroundView=city&currentCityId={}&templateView=branchOffice&actionRequest={}&ajax=1'.format(oferta['destinationCityId'], oferta['cityId'], oferta['position'], oferta['type'], oferta['resource'], oferta['cityId'], s.token())
	data = s.post(url)
	html = json.loads(data, strict=False)[1][1][1]
	hits = re.findall(r'"tradegood(\d)Price"\s*value="(\d+)', html)
	for hit in hits:
		data_dict['tradegood{}Price'.format(hit[0])] = int(hit[1])
		data_dict['cargo_tradegood{}'.format(hit[0])] = 0
	hit = re.search(r'"resourcePrice"\s*value="(\d+)', html)
	if hit:
		data_dict['resourcePrice'] = int(hit.group(1))
		data_dict['cargo_resource'] = 0
	resource = oferta['resource']
	if resource == 'resource':
		data_dict['cargo_resource'] = cantidad
	else:
		data_dict['cargo_tradegood{}'.format(resource)] = cantidad
	s.post(payloadPost=data_dict)
	msg = _('Compro {} a {} de {}').format(addPuntos(cantidad), oferta['ciudadDestino'], oferta['jugadorAComprar'])
	sendToBotDebug(msg, debugON_comprarRecursos)

def do_it(s, ciudad, ofertas, cantidadAComprar):
	while True:
		for oferta in ofertas:
			if cantidadAComprar == 0:
				return
			if oferta['cantidadDisponible'] == 0:
				continue
			barcosDisp = esperarLlegada(s)
			capacidad  = barcosDisp * 500
			comprable_max = capacidad if capacidad < cantidadAComprar else cantidadAComprar
			compra = comprable_max if oferta['cantidadDisponible'] > comprable_max else oferta['cantidadDisponible']
			cantidadAComprar -= compra
			oferta['cantidadDisponible'] -= compra
			buy(s, ciudad, oferta, compra)
			break # vuelvo a empezar desde el principio



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import hashlib
import base64
from Crypto.Cipher import AES

class AESCipher:

	def __init__( self, creds ):
		self.key = hashlib.sha256( creds['name'].encode('utf-8') + creds['password'].encode('utf-8') ).digest()
		for i in range(0xfff):
			self.key = hashlib.sha256( self.key ).digest()

	def encrypt( self, plaintext ):
		cipher          = AES.new(self.key, AES.MODE_EAX)
		nonce           = cipher.nonce
		ciphertext, tag = cipher.encrypt_and_digest( plaintext.encode() )
		return base64.b64encode( nonce + tag + ciphertext ).decode('utf-8')

	def decrypt( self, ciphertext ):
		ciphertext = base64.b64decode(ciphertext)
		nonce      = ciphertext[:16]
		ciphertext = ciphertext[16:]
		tag        = ciphertext[:16]
		ciphertext = ciphertext[16:]
		cipher     = AES.new(self.key, AES.MODE_EAX, nonce=nonce)
		return cipher.decrypt_and_verify( ciphertext, tag ).decode('utf-8')



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import gettext
from decimal import *
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import *
from ikabot.helpers.naval import *
from ikabot.helpers.recursos import *
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.pedirInfo import getIdsDeCiudades

t = gettext.translation('getStatus', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

getcontext().prec = 30

def getStatus(s):
	banner()
	tipoCiudad = [bcolors.ENDC, bcolors.HEADER, bcolors.STONE, bcolors.BLUE, bcolors.WARNING]
	ids, ciudades = getIdsDeCiudades(s)
	print(_('Barcos {:d}/{:d}').format(getBarcosDisponibles(s), getBarcosTotales(s)))
	for unId in ids:
		html = s.get(urlCiudad + unId)
		ciudad = getCiudad(html)
		(wood, good, typeGood) = getProduccion(s, unId)
		print('\033[1m' + tipoCiudad[int(typeGood)] + ciudad['cityName'] + tipoCiudad[0])
		max = getRecursosDisponibles(html)
		capacidadDeAlmacenamiento = getCapacidadDeAlmacenamiento(html)
		crecursos = []
		for i in range(0,5):
			if max[i] == capacidadDeAlmacenamiento:
				crecursos.append(bcolors.RED)
			else:
				crecursos.append(bcolors.ENDC)
		print(_('Almacenamiento:'))
		print(addPuntos(capacidadDeAlmacenamiento))
		print(_('Recursos:'))
		print(_('Madera {1}{2}{0} Vino {3}{4}{0} Marmol {5}{6}{0} Cristal {7}{8}{0} Azufre {9}{10}{0}').format(bcolors.ENDC, crecursos[0], addPuntos(max[0]), crecursos[1], addPuntos(max[1]), crecursos[2], addPuntos(max[2]), crecursos[3], addPuntos(max[3]), crecursos[4], addPuntos(max[4])))
		consumoXhr = getConsumoDeVino(html)
		tipo = tipoDeBien[typeGood]
		print(_('Producción:'))
		print(_('Madera:{} {}:{}').format(addPuntos(wood*3600), tipo, addPuntos(good*3600)))
		if consumoXhr == 0:
			print(_('{}{}No se consume vino!{}').format(bcolors.RED, bcolors.BOLD, bcolors.ENDC))
		elif typeGood == 1 and (good*3600) > consumoXhr:
			print(_('Hay vino para:\n∞'))
		else:
			consumoXseg = Decimal(consumoXhr) / Decimal(3600)
			segsRestantes = Decimal(int(max[1])) / Decimal(consumoXseg)
			texto = diasHorasMinutos(segsRestantes)
			print(_('Hay vino para:\n{}').format(texto))
		for edificio in [ edificio for edificio in ciudad['position'] if edificio['name'] != 'empty' ]:
			if edificio['isMaxLevel'] is True:
				color = bcolors.BLACK
			elif edificio['canUpgrade'] is True:
				color = bcolors.GREEN
			else:
				color = bcolors.RED
			level = edificio['level']
			if int(level) < 10:
				level = ' ' + level
			if edificio['isBusy'] is True:
				level = level + '+'
			print(_('lv:{}\t{}{}{}').format(level, color, edificio['name'], bcolors.ENDC))
		enter()
		print('')

def getProduccion(s, idCiudad):
	prod = s.post(payloadPost={'action': 'header', 'function': 'changeCurrentCity', 'actionRequest': s.token(), 'cityId': idCiudad, 'ajax': '1'}) 
	wood = Decimal(re.search(r'"resourceProduction":([\d|\.]+),', prod).group(1))
	good = Decimal(re.search(r'"tradegoodProduction":([\d|\.]+),', prod).group(1))
	typeGood = int(re.search(r'"producedTradegood":"([\d|\.]+)"', prod).group(1))
	return (wood, good, typeGood)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import traceback
import re
import json
from ikabot.config import *
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.process import forkear
from ikabot.helpers.gui import enter
from ikabot.helpers.varios import diasHorasMinutos
from ikabot.helpers.botComm import *

t = gettext.translation('alertarAtaques',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def alertarAtaques(s):
	if botValido(s) is False:
		return
	print(_('Se buscarán ataques cada 20 minutos.'))
	enter()

	forkear(s)
	if s.padre is True:
		return

	info = _('\nEspero por ataques cada 20 minutos\n')
	setInfoSignal(s, info)
	try:
		do_it(s)
	except:
		msg = _('Error en:\n{}\nCausa:\n{}').format(info, traceback.format_exc())
		sendToBot(msg)
	finally:
		s.logout()

def do_it(s):
	conocidos = []
	while True:
		html = s.get()
		idCiudad = re.search(r'currentCityId:\s(\d+),', html).group(1)
		url = 'view=militaryAdvisor&oldView=city&oldBackgroundView=city&backgroundView=city&currentCityId={}&actionRequest={}&ajax=1'.format(idCiudad, s.token())
		posted = s.post(url)
		postdata = json.loads(posted, strict=False)
		militaryMovements = postdata[1][1][2]['viewScriptParams']['militaryAndFleetMovements']
		tiempoAhora = int(postdata[0][1]['time'])
		actuales = []
		for militaryMovement in militaryMovements:
			if militaryMovement['isHostile']:
				id = militaryMovement['event']['id']
				actuales.append(id)
				if id not in conocidos:
					conocidos.append(id)
					missionText = militaryMovement['event']['missionText']
					origin = militaryMovement['origin']
					target = militaryMovement['target']
					cantidadTropas = militaryMovement['army']['amount']
					cantidadFlotas = militaryMovement['fleet']['amount']
					tiempoFaltante = int(militaryMovement['eventTime']) - tiempoAhora
					msg  = _('-- ALERTA --\n')
					msg += missionText + '\n'
					msg += _('de la ciudad {} de {}\n').format(origin['name'], origin['avatarName'])
					msg += _('a {}\n').format(target['name'])
					msg += _('{} unidades\n').format(cantidadTropas)
					msg += _('{} flotas\n').format(cantidadFlotas)
					msg += _('llegada en: {}').format(diasHorasMinutos(tiempoFaltante))
					sendToBot(msg)
		for id in list(conocidos):
			if id not in actuales:
				conocidos.remove(id)
		time.sleep(20 * 60)
